@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003 - 2010 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -41,13 +41,15 @@
 # include "magick/nt_feature.h"
 #endif
 #include "magick/blob.h"
-#include "magick/color.h"
+#include "magick/color_lookup.h"
+#include "magick/command.h"
 #include "magick/constitute.h"
 #include "magick/delegate.h"
 #include "magick/log.h"
 #include "magick/magic.h"
 #include "magick/magick.h"
 #include "magick/module.h"
+#include "magick/monitor.h"
 #include "magick/pixel_cache.h"
 #include "magick/random.h"
 #include "magick/registry.h"
@@ -78,15 +80,25 @@ static void
 static SemaphoreInfo
   *magick_semaphore = (SemaphoreInfo *) NULL;
 
+#if defined(SupportMagickModules)
+static SemaphoreInfo
+  *module_semaphore = (SemaphoreInfo *) NULL;
+#endif /* #if defined(SupportMagickModules) */
+
 static MagickInfo
   *magick_list = (MagickInfo *) NULL;
 
+static unsigned int panic_signal_handler_call_count = 0;
+static unsigned int quit_signal_handler_call_count = 0;
+
 static MagickInitializationState MagickInitialized = InitDefault;
 static CoderClass MinimumCoderClass = UnstableCoderClass;
 
 static void DestroyMagickInfo(MagickInfo** magick_info);
 static void DestroyMagickInfoList(void);
 
+static MagickPassFail InitializeMagickInfoList(void);
+
 /*
   Block size to use when accessing filesystem.
 
@@ -125,7 +137,15 @@ MagickSetFileSystemBlockSize(const size_t block_size)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  DestroyMagick() destroys the GraphicsMagick environment.
+%  DestroyMagick() destroys the GraphicsMagick environment, releasing all
+%  allocated semaphores, memory, and temporary files.  This function
+%  should be invoked in the primary (original) thread of the application's
+%  process while shutting down, and only after any threads which might be
+%  using GraphicsMagick functions have terminated.  Since GraphicsMagick
+%  uses threads internally via OpenMP, it is also necessary for any function
+%  calls into GraphicsMagick to have already returned so that OpenMP worker
+%  threads are quiesced and won't be accessing any semaphores or data
+%  structures which are destroyed by this function.
 %
 %  The format of the DestroyMagick function is:
 %
@@ -133,11 +153,23 @@ MagickSetFileSystemBlockSize(const size_t block_size)
 %
 %
 */
-MagickExport void DestroyMagick(void)
+MagickExport void
+DestroyMagick(void)
 {
-  if (MagickInitialized == InitUninitialized)
-    return;
+  /* Acquire destruction lock */
+  SPINLOCK_WAIT;
 
+  if (MagickInitialized == InitUninitialized)
+    {
+      /* Release destruction lock */
+      SPINLOCK_RELEASE;
+      return;
+    }
+
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+			"Destroy Magick");
+
+  MagickDestroyCommandInfo();   /* Command parser */
 #if defined(HasX11)
   MagickXDestroyX11Resources();
 #endif
@@ -160,8 +192,11 @@ MagickExport void DestroyMagick(void)
   DestroyLogInfo();             /* Logging configuration */
   DestroySemaphore();           /* Semaphores framework */
 
+  /* Now uninitialized */
   MagickInitialized=InitUninitialized;
 
+  /* Release destruction lock */
+  SPINLOCK_RELEASE;
 }
 /*
   Destroy MagickInfo structure.
@@ -185,7 +220,8 @@ MagickExport void DestroyMagick(void)
 %      void DestroyMagickInfo(MagickInfo** magick_info)
 %
 */
-static void DestroyMagickInfo(MagickInfo** magick_info)
+static void
+DestroyMagickInfo(MagickInfo** magick_info)
 {
   MagickInfo
     *p;
@@ -221,7 +257,8 @@ static void DestroyMagickInfo(MagickInfo** magick_info)
 %      void DestroyMagickInfoList(void)
 %
 */
-static void DestroyMagickInfoList(void)
+static void
+DestroyMagickInfoList(void)
 {
   MagickInfo
     *magick_info;
@@ -241,7 +278,6 @@ static void DestroyMagickInfoList(void)
     At this point, the list should be empty, but check for remaining
     entries anyway.
   */
-  AcquireSemaphoreInfo(&magick_semaphore);
 #if defined(BuildMagickModules)
   if (magick_list != (MagickInfo *) NULL)
     (void) printf("Warning: module registrations are still present!\n");
@@ -258,8 +294,11 @@ static void DestroyMagickInfoList(void)
     DestroyMagickInfo(&magick_info);
   }
   magick_list=(MagickInfo *) NULL;
-  LiberateSemaphoreInfo(&magick_semaphore);
   DestroySemaphoreInfo(&magick_semaphore);
+
+#if defined(SupportMagickModules)
+  DestroySemaphoreInfo(&module_semaphore);
+#endif /* #if defined(SupportMagickModules) */
 }
 
 /*
@@ -289,18 +328,18 @@ static void DestroyMagickInfoList(void)
 %
 %
 */
-MagickExport const char *GetImageMagick(const unsigned char *magick,
-  const size_t length)
+MagickExport const char *
+GetImageMagick(const unsigned char *magick,const size_t length)
 {
   register MagickInfo
     *p;
 
   assert(magick != (const unsigned char *) NULL);
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
   for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
     if (p->magick && p->magick(magick,length))
       break;
-  LiberateSemaphoreInfo(&magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
   if (p != (MagickInfo *) NULL)
     return(p->name);
   return((char *) NULL);
@@ -324,6 +363,9 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
 %  may be altered while the list is being traversed. If the list must be
 %  traversed, access it via the GetMagickInfoArray function instead.
 %
+%  If GraphicsMagick has not been initialized via InitializeMagick()
+%  then this function will not work.
+%
 %  The format of the GetMagickInfo method is:
 %
 %     const MagickInfo *GetMagickInfo(const char *name,ExceptionInfo *exception)
@@ -336,12 +378,13 @@ MagickExport const char *GetImageMagick(const unsigned char *magick,
 %
 %
 */
-static MagickInfo *GetMagickInfoEntryLocked(const char *name)
+static MagickInfo *
+GetMagickInfoEntryLocked(const char *name)
 {
   register MagickInfo
     *p;
 
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
 
   p=magick_list;
 
@@ -368,58 +411,51 @@ static MagickInfo *GetMagickInfoEntryLocked(const char *name)
           }
     }
 
-  LiberateSemaphoreInfo(&magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
 
   return p;
 }
-MagickExport const MagickInfo *GetMagickInfo(const char *name,
-  ExceptionInfo *ARGUNUSED(exception))
+MagickExport const MagickInfo *
+GetMagickInfo(const char *name,ExceptionInfo *ARGUNUSED(exception))
 {
-#if !defined(BuildMagickModules)
-  if ((magick_list == (MagickInfo *) NULL) &&
-      (GetMagickInfoEntryLocked(NULL) == (MagickInfo *) NULL))
-    {
-      /*
-        Register all static modules
-      */
-      RegisterStaticModules();
-    }
-#endif /* !defined(BuildMagickModules) */
-
-  /*
-    If name is NULL, then return head of list (as it currently
-    exists).
-  */
-  if (name == (const char *) NULL)
-    return GetMagickInfoEntryLocked(name);
+  const MagickInfo
+    *magick_info=(const MagickInfo *) NULL;
 
 #if defined(SupportMagickModules)
-  /*
-    Load module aliases (does nothing if already loaded)
-  */
-  (void) GetModuleInfo((char *) NULL,exception);
-
-  if (name[0] == '*')
+  if ((name != (const char *) NULL) &&
+      (name[0] != '\0'))
     {
-      /*
-        If all modules are requested, then use OpenModules to load all
-        modules.
-      */
-      (void) OpenModules(exception);
-    }
-  else
-    {
-      /*
-        Try to load a supporting module.
-      */
-      (void) OpenModule(name,exception);
+      LockSemaphoreInfo(module_semaphore);
+      if (name[0] == '*')
+	{
+	  /*
+	    If all modules are requested, then use OpenModules to load all
+	    modules.
+	  */
+	  (void) OpenModules(exception);
+	}
+      else
+	{
+	  magick_info=GetMagickInfoEntryLocked(name);
+	  if (magick_info == (const MagickInfo *) NULL)
+	    {
+	      /*
+		Try to load a supporting module.
+	      */
+	      (void) OpenModule(name,exception);
+	    }
+	}
+      UnlockSemaphoreInfo(module_semaphore);
     }
 #endif /* #if defined(SupportMagickModules) */
 
   /*
     Return whatever we've got
   */
-  return GetMagickInfoEntryLocked(name);
+  if (magick_info == (const MagickInfo *) NULL)
+    magick_info=GetMagickInfoEntryLocked(name);
+
+  return magick_info;
 }
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -454,7 +490,8 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
 /*
   Compare two MagickInfo structures based on their name
 */
-static int MagickInfoCompare(const void *x, const void *y)
+static int
+MagickInfoCompare(const void *x, const void *y)
 {
   const MagickInfo
     *xx=*((const MagickInfo **) x),
@@ -462,7 +499,8 @@ static int MagickInfoCompare(const void *x, const void *y)
 
   return (strcmp(xx->name, yy->name));
 }
-MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
+MagickExport MagickInfo **
+GetMagickInfoArray(ExceptionInfo *exception)
 {
   MagickInfo
     **array;
@@ -484,9 +522,9 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
   */
   (void) GetMagickInfo("*",exception);
   if (!magick_list)
-    return 0;
+    return ((MagickInfo **) NULL);
 
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
 
   list=magick_list;
 
@@ -499,13 +537,12 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
   /*
     Allocate array memory
   */
-  array=MagickAllocateMemory(MagickInfo **,sizeof(MagickInfo *)*(entries+1));
+  array=MagickAllocateArray(MagickInfo **,sizeof(MagickInfo *),(entries+1));
   if (!array)
     {
       ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,0);
-      return False;
+      return ((MagickInfo **) NULL);
     }
-  (void) memset((void **)array,0,sizeof(MagickInfo *)*(entries+1));
 
   /*
     Add entries to array
@@ -513,8 +550,9 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
   i=0;
   for (p=list; p != 0; p=p->next)
     array[i++]=p;
+  array[i]=(MagickInfo *) NULL;
 
-  LiberateSemaphoreInfo(&magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
 
   /*
     Sort array entries
@@ -537,7 +575,11 @@ MagickExport MagickInfo **GetMagickInfoArray(ExceptionInfo *exception)
 %
 %  InitializeMagick() initializes the GraphicsMagick environment.
 %  InitializeMagick() MUST be invoked by the using program before making
-%  use of GraphicsMagick functions or else the library may be unstable.
+%  use of GraphicsMagick functions or else the library will be unusable.
+%
+%  This function should be invoked in the primary (original) thread of the
+%  application's process, and before starting any OpenMP threads, as part
+%  of program initialization.
 %
 %  The format of the InitializeMagick function is:
 %
@@ -567,12 +609,16 @@ typedef RETSIGTYPE Sigfunc(int);
 # define SIG_DFL (Sigfunc *)0
 #endif
 
+static RETSIGTYPE MagickPanicSignalHandler(int signo);
+static RETSIGTYPE MagickSignalHandler(int signo);
+
 /*
   Signal function which prevents interrupted system calls from
   automatically being restarted. From W. Richard Stevens "Advanced
   Programming in the UNIX Environment", Chapter 10.14.
 */
-static Sigfunc *MagickSignal(int signo, Sigfunc *func)
+static Sigfunc *
+MagickSignal(int signo, Sigfunc *func)
 {
 #if defined(HAVE_SIGACTION) && defined(HAVE_SIGEMPTYSET)
   struct sigaction
@@ -602,7 +648,8 @@ static Sigfunc *MagickSignal(int signo, Sigfunc *func)
   If an API user registers its own signal hander, then it is responsible
   for invoking DestroyMagick when a signal is received.
 */
-static Sigfunc *MagickCondSignal(int signo, Sigfunc *func)
+static Sigfunc *
+MagickCondSignal(int signo, Sigfunc *func)
 {
   Sigfunc *
     o_handler;
@@ -626,7 +673,8 @@ static Sigfunc *MagickCondSignal(int signo, Sigfunc *func)
   Signal handler which does nothing.
 */
 #if 0
-static RETSIGTYPE MagickIgnoreSignalHandler(int signo)
+static RETSIGTYPE
+MagickIgnoreSignalHandler(int signo)
 {
   fprintf(stderr,"Caught ignored signal %d\n", signo);
 }
@@ -649,54 +697,84 @@ static RETSIGTYPE MagickIgnoreSignalHandler(int signo)
   While support for signal is definitely broken under Windows, the good
   news is that it seems to be unlikely to generate a signal we care about.
 */
-static RETSIGTYPE MagickPanicSignalHandler(int signo)
+
+static RETSIGTYPE
+MagickPanicSignalHandler(int signo)
 {
-  /* fprintf(stderr,"Caught panic signal %d\n", signo); */
+  ARG_NOT_USED(signo);
+
+  panic_signal_handler_call_count++;
 
   /*
-    Restore default handling for the signal
+    Only handle signal one time.
   */
-  (void) MagickSignal(signo,SIG_DFL);
+  if (1 == panic_signal_handler_call_count)
+    {
+      if (MagickInitialized == InitInitialized)
+	{
+	  /*
+	    Release persistent resources
+	  */
+	  PurgeTemporaryFiles();
+	}
 
-  /*
-    Release resources
-  */
-  DestroyTemporaryFiles();
-
-  /*
-    Raise signal again to invoke default handler
-    This may cause a core dump or immediate exit.
-  */
-#if defined(HAVE_RAISE)
-  (void) fflush(stdout);
-  (void) raise(signo);
-#endif
-
-  SignalHandlerExit(signo);
+      /*
+	Call abort so that we quit with core dump.
+      */
+      abort();
+    }
 }
-static RETSIGTYPE MagickSignalHandler(int signo)
+
+static MagickBool QuitProgressMonitor(const char *task,
+				      const magick_int64_t quantum,
+				      const magick_uint64_t span,
+				      ExceptionInfo *exception)
+{
+  ARG_NOT_USED(task);
+  ARG_NOT_USED(quantum);
+  ARG_NOT_USED(span);
+
+  /* Report an error message */
+  if (exception->severity < FatalErrorException)
+    ThrowException(exception,MonitorFatalError,UserRequestedTerminationBySignal,0);
+
+  return MagickFail;
+}
+
+
+static RETSIGTYPE
+MagickSignalHandler(int signo)
 {
   /* fprintf(stderr,"Caught signal %d\n", signo); */
 
-  /*
-    Restore default handling for the signal
-  */
-  (void) MagickSignal(signo,SIG_DFL);
+  quit_signal_handler_call_count++;
 
   /*
-    Release resources
+    Only handle signal one time.
   */
-  DestroyMagick();
+  if (1 == quit_signal_handler_call_count)
+    {
+      if (MagickInitialized == InitInitialized)
+	{
 
-  /*
-    Raise signal again to invoke default handler
-    This may cause a core dump or immediate exit.
-  */
-#if defined(HAVE_RAISE)
-  (void) raise(signo);
-#endif
+	  /*
+	    Set progress monitor handler to one which always returns
+	    MagickFail.
+	  */
+	  (void) SetMonitorHandler(QuitProgressMonitor);
 
-  SignalHandlerExit(signo);
+	  /*
+	    Release persistent resources
+	  */
+	  PurgeTemporaryFiles();
+	}
+
+      /*
+	Invoke _exit(signo) (or equivalent) which avoids invoking
+	registered atexit() functions.
+      */
+      SignalHandlerExit(signo);
+    }
 }
 
 /*
@@ -705,7 +783,8 @@ static RETSIGTYPE MagickSignalHandler(int signo)
    filesystem
  */
 #if !defined(UseInstalledMagick)
-static unsigned int IsValidFilesystemPath(const char *path)
+static unsigned int
+IsValidFilesystemPath(const char *path)
 {
   if ((path != (const char *) NULL) && (*path != '\0'))
     {
@@ -737,7 +816,8 @@ static unsigned int IsValidFilesystemPath(const char *path)
 /*
   Try and figure out the path and name of the client
  */
-MagickExport void InitializeMagickClientPathAndName(const char *ARGUNUSED(path))
+MagickExport void
+InitializeMagickClientPathAndName(const char *path)
 {
 #if !defined(UseInstalledMagick)
   const char
@@ -808,13 +888,16 @@ MagickExport void InitializeMagickClientPathAndName(const char *ARGUNUSED(path))
       (void) DefineClientPathAndName(execution_path);
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),"Valid path \"%s\"",spath);
     }
+#else
+  ARG_NOT_USED(path);
 #endif
 }
 
 /*
   Establish signal handlers for common signals
 */
-MagickExport void InitializeMagickSignalHandlers(void)
+MagickExport void
+InitializeMagickSignalHandlers(void)
 {
 #if 0
   /* termination of child process */
@@ -827,11 +910,11 @@ MagickExport void InitializeMagickSignalHandlers(void)
 #if defined(SIGHUP)
   (void) MagickCondSignal(SIGHUP,MagickSignalHandler);
 #endif
-  /* interrupt (rubout), default terminate */
+  /* interrupt (CONTROL-c), default terminate */
 #if defined(SIGINT) && !defined(MSWINDOWS)
   (void) MagickCondSignal(SIGINT,MagickSignalHandler);
 #endif
-  /* quit (ASCII FS), default terminate with core */
+  /* quit (CONTROL-\), default terminate with core */
 #if defined(SIGQUIT)
   (void) MagickCondSignal(SIGQUIT,MagickPanicSignalHandler);
 #endif
@@ -857,23 +940,30 @@ MagickExport void InitializeMagickSignalHandlers(void)
 #endif
 }
 
-MagickExport void InitializeMagick(const char *path)
+MagickExport void
+InitializeMagick(const char *path)
 {
+  /*
+    NOTE: This routine sets up the path to the client which needs to
+    be determined before almost anything else works right. This also
+    includes logging!!! So we can't start logging until the path is
+    actually saved. As soon as we know what the path is we make the
+    same call to DefineClientSettings to set it up. Please make sure
+    that this rule is followed in any future updates the this code!!!
+  */
+
   const char
     *p;
 
-  /* NOTE: This routine sets up the path to the client which needs to
-     be determined before almost anything else works right. This also
-     includes logging!!! So we can't start logging until the path is
-     actually saved. As soon as we know what the path is we make the
-     same call to DefineClientSettings to set it up. Please make sure
-     that this rule is followed in any future updates the this code!!!
-  */
-  if (MagickInitialized == InitInitialized)
-    return;
+  /* Acquire initialization lock */
   SPINLOCK_WAIT;
-  MagickInitialized=InitInitialized;
-  SPINLOCK_RELEASE;
+
+  if (MagickInitialized == InitInitialized)
+    {
+      /* Release initialization lock */
+      SPINLOCK_RELEASE;
+      return;
+    }
   
 #if defined(MSWINDOWS)
 # if defined(_DEBUG) && !defined(__BORLANDC__)
@@ -889,13 +979,13 @@ MagickExport void InitializeMagick(const char *path)
   }
 # endif /* defined(_DEBUG) */
 #endif /* defined(MSWINDOWS) */
-  
-  (void) setlocale(LC_ALL,"");
-  (void) setlocale(LC_NUMERIC,"C");
 
   /* Initialize semaphores */
   InitializeSemaphore();
 
+  /* Initialize logging */
+  InitializeLogInfo();
+
   /* Seed the random number generator */
   srand(MagickRandNewSeed());
 
@@ -909,6 +999,9 @@ MagickExport void InitializeMagick(const char *path)
   if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
     (void) SetLogEventMask(p);
 
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+			"Initialize Magick");
+
   /*
     Set the filesystem block size.
   */
@@ -917,7 +1010,7 @@ MagickExport void InitializeMagick(const char *path)
       block_size=16384;
     
     if ((p=getenv("MAGICK_IOBUF_SIZE")) != (const char *) NULL)
-      block_size = (size_t) atol(p);
+      block_size = (size_t) MagickAtoL(p);
     
     MagickSetFileSystemBlockSize(block_size);
   }
@@ -933,53 +1026,82 @@ MagickExport void InitializeMagick(const char *path)
   if (GetClientName() == (const char *) NULL)
     DefineClientName(path);
 
-  /*
-    Set logging flags using the value of MAGICK_DEBUG if it is set in
-    the environment.
-  */
-  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
-    (void) SetLogEventMask(p);
-
-  /*
-    Establish signal handlers for common signals
-  */
-  InitializeMagickSignalHandlers();
-
-  /*
-    Compute memory allocation and memory map resource limits based
-    on available memory
-  */
-  /* NOTE: This call does logging, so you can not make it before the path
-     to the client is setup */
-  InitializeMagickResources();
-
-  /* Initialize magick registry */
-  InitializeMagickRegistry();
-  
   /*
     Adjust minimum coder class if requested.
   */
   if ((p=getenv("MAGICK_CODER_STABILITY")) != (const char *) NULL)
     {
       if (LocaleCompare(p,"UNSTABLE") == 0)
-        MinimumCoderClass=UnstableCoderClass;
+	MinimumCoderClass=UnstableCoderClass;
       else if (LocaleCompare(p,"STABLE") == 0)
-        MinimumCoderClass=StableCoderClass;
+	MinimumCoderClass=StableCoderClass;
       else if (LocaleCompare(p,"PRIMARY") == 0)
-        MinimumCoderClass=PrimaryCoderClass;
+	MinimumCoderClass=PrimaryCoderClass;
     }
 
-  /*
-    Initialize module loader
-  */
-#if defined(SupportMagickModules)
-  InitializeMagickModules();
-#endif /* defined(SupportMagickModules) */
+  InitializeMagickSignalHandlers(); /* Signal handlers */
+  InitializeTemporaryFiles();       /* Temporary files */
+  InitializeMagickResources();      /* Resources */
+  InitializeMagickRegistry();       /* Image/blob registry */
+  InitializeConstitute();           /* Constitute semaphore */
+  InitializeMagickInfoList();       /* Coder registrations + modules */
+  InitializeMagicInfo();            /* File format detection */
+  InitializeTypeInfo();             /* Font information */
+  InitializeDelegateInfo();         /* External delegate information */
+  InitializeColorInfo();            /* Color database */
+  MagickInitializeCommandInfo();    /* Command parser */
 
   /* Let's log the three important setting as we exit this routine */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Path: \"%s\" Name: \"%s\" Filename: \"%s\"",
-      GetClientPath(),GetClientName(),GetClientFilename());
+			"Path: \"%s\" Name: \"%s\" Filename: \"%s\"",
+			GetClientPath(),GetClientName(),GetClientFilename());
+
+  /* Now initialized */
+  MagickInitialized=InitInitialized;
+
+  /* Release initialization lock */
+  SPINLOCK_RELEASE;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   I n i t i a l i z e M a g i c k I n f o L i s t                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method InitializeMagickInfoList initializes the magick info facility
+%
+%  The format of the InitializeMagickInfoList method is:
+%
+%      MagickPassFail InitializeMagickInfoList(void)
+%
+%
+*/
+MagickPassFail
+InitializeMagickInfoList(void)
+{
+  assert(magick_semaphore == (SemaphoreInfo *) NULL);
+  magick_semaphore=AllocateSemaphoreInfo();
+
+#if defined(SupportMagickModules)
+  assert(module_semaphore == (SemaphoreInfo *) NULL);
+  module_semaphore=AllocateSemaphoreInfo();
+#endif /* #if defined(SupportMagickModules) */
+
+#if !defined(BuildMagickModules)
+  RegisterStaticModules();          /* Register all static modules */
+#endif /* !defined(BuildMagickModules) */
+
+#if defined(SupportMagickModules)
+  InitializeMagickModules();        /* Module loader */
+#endif /* defined(SupportMagickModules) */
+
+  return MagickPass;
 }
 
 /*
@@ -1009,7 +1131,8 @@ MagickExport void InitializeMagick(const char *path)
 %
 %
 */
-MagickExport MagickBool IsMagickConflict(const char *magick)
+MagickExport MagickBool
+IsMagickConflict(const char *magick)
 {
   assert(magick != (char *) NULL);
 #if defined(MSWINDOWS) || defined(__CYGWIN__)
@@ -1043,7 +1166,8 @@ MagickExport MagickBool IsMagickConflict(const char *magick)
 %
 %
 */
-MagickExport MagickPassFail ListMagickInfo(FILE *file,ExceptionInfo *exception)
+MagickExport MagickPassFail
+ListMagickInfo(FILE *file,ExceptionInfo *exception)
 {
 
   MagickInfo
@@ -1131,7 +1255,8 @@ MagickExport MagickPassFail ListMagickInfo(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-MagickExport MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
+MagickExport MagickPassFail
+ListModuleMap(FILE *file,ExceptionInfo *exception)
 {
   MagickInfo
     **magick_array;
@@ -1155,11 +1280,10 @@ MagickExport MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
      {
        if (LocaleCompare(magick_array[i]->name,magick_array[i]->module) != 0)
          {
-/*            if (i != 0) */
-/*              (void) fprintf(file, "\n"); */
-/*            (void) fprintf(file, "  <!-- %s -->\n",magick_array[i]->description); */
            (void) fprintf(file, "  <module magick=\"%s\" name=\"%s\" />\n",
-                          magick_array[i]->name, magick_array[i]->module);
+                          magick_array[i]->name,
+			  (magick_array[i]->module == NULL ? "(null)" :
+			   magick_array[i]->module));
          }
      }
    (void) fprintf(file, "</modulemap>\n");
@@ -1196,7 +1320,8 @@ MagickExport MagickPassFail ListModuleMap(FILE *file,ExceptionInfo *exception)
 %
 %
 */
-MagickExport char *MagickToMime(const char *magick)
+MagickExport char *
+MagickToMime(const char *magick)
 {
   typedef struct _MediaType
   {
@@ -1278,7 +1403,8 @@ MagickExport char *MagickToMime(const char *magick)
 %    o magick_info: The magick info.
 %
 */
-MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
+MagickExport MagickInfo *
+RegisterMagickInfo(MagickInfo *magick_info)
 {
   assert(magick_info != (MagickInfo *) NULL);
   assert(magick_info->signature == MagickSignature);
@@ -1296,13 +1422,13 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
       /*
         Add to front of list.
       */
-      AcquireSemaphoreInfo(&magick_semaphore);
+      LockSemaphoreInfo(magick_semaphore);
       magick_info->previous=(MagickInfo *) NULL;
       magick_info->next=magick_list;
       if (magick_info->next != (MagickInfo *) NULL)
         magick_info->next->previous=magick_info;
       magick_list=magick_info;
-      LiberateSemaphoreInfo(&magick_semaphore);
+      UnlockSemaphoreInfo(magick_semaphore);
       return(magick_info);
     }
 
@@ -1341,7 +1467,8 @@ MagickExport MagickInfo *RegisterMagickInfo(MagickInfo *magick_info)
 %
 %
 */
-MagickExport MagickInfo *SetMagickInfo(const char *name)
+MagickExport MagickInfo *
+SetMagickInfo(const char *name)
 {
   MagickInfo
     *magick_info;
@@ -1393,7 +1520,8 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
 %      looking for.
 %
 */
-MagickExport MagickPassFail UnregisterMagickInfo(const char *name)
+MagickExport MagickPassFail
+UnregisterMagickInfo(const char *name)
 {
   MagickInfo
     *magick_info;
@@ -1406,7 +1534,7 @@ MagickExport MagickPassFail UnregisterMagickInfo(const char *name)
 
   assert(name != (const char *) NULL);
   status=MagickFail;
-  AcquireSemaphoreInfo(&magick_semaphore);
+  LockSemaphoreInfo(magick_semaphore);
   for (p=magick_list; p != (MagickInfo *) NULL; p=p->next)
   {
     if (LocaleCompare(p->name,name) != 0)
@@ -1422,6 +1550,6 @@ MagickExport MagickPassFail UnregisterMagickInfo(const char *name)
     status=MagickPass;
     break;
   }
-  LiberateSemaphoreInfo(&magick_semaphore);
+  UnlockSemaphoreInfo(magick_semaphore);
   return(status);
 }
