@@ -1,4 +1,11 @@
 /*
+ * Copyright (C) 2003 - 2011 GraphicsMagick Group
+ * Copyright (C) 2003 ImageMagick Studio
+ * Copyright 1991-1999 E. I. du Pont de Nemours and Company
+ *
+ * This program is covered by multiple licenses, which are described in
+ * Copyright.txt. You should have received a copy of Copyright.txt with this
+ * package; otherwise see http://www.graphicsmagick.org/www/Copyright.html.
  *
  * Test BLOB operations via write/read/write/read sequence to detect
  * any data corruption problems. This does not verify that the image is
@@ -42,11 +49,12 @@ int main ( int argc, char **argv )
     columns = 0,
     pause = 0;
 
-  MagickBool
-    check = MagickTrue;
+  unsigned long
+    original_frames;
 
-  double
-    fuzz_factor = 0;
+  MagickBool
+    check = MagickTrue,
+    check_for_added_frames = MagickTrue;
 
   ImageInfo
     *imageInfo;
@@ -108,9 +116,11 @@ int main ( int argc, char **argv )
                   exit_status = 1;
                   goto program_exit;
                 }
-              if(imageInfo->depth != 8 && imageInfo->depth != 16 && imageInfo->depth != 32)
+              if(imageInfo->depth != 8 && imageInfo->depth != 16 &&
+		 imageInfo->depth != 32)
                 {
-                  (void) printf("-depth (%ld) not 8, 16, or 32\n", imageInfo->depth);
+                  (void) printf("-depth (%ld) not 8, 16, or 32\n",
+				imageInfo->depth);
                   (void) fflush(stdout);
                   exit_status = 1;
                   goto program_exit;
@@ -153,7 +163,9 @@ int main ( int argc, char **argv )
   if (arg != argc-2)
     {
       (void) printf("arg=%d, argc=%d\n", arg, argc);
-      (void) printf ( "Usage: %s [-compress algorithm -debug events -depth integer -log format -nocheck -size geometry -verbose] infile format\n", argv[0] );
+      (void) printf ( "Usage: %s [-compress algorithm -debug events -depth "
+		      "integer -log format -nocheck -size geometry -verbose] "
+		      "infile format\n", argv[0] );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -163,10 +175,23 @@ int main ( int argc, char **argv )
   arg++;
   (void) strncpy( format, argv[arg], MaxTextExtent-1 );
 
-  /*   for (arg=0; arg < argc; arg++) */
-  /*     (void) printf("%s ", argv[arg]); */
-  /*   (void) printf("\n"); */
-  /*   (void) fflush(stdout); */
+  magick_info=GetMagickInfo(format,&exception);
+  if (magick_info == (MagickInfo *) NULL)
+    {
+      fprintf(stderr, "No support for \"%s\" format.\n",format);
+      exit(1);
+    }
+
+  /*
+    Some formats intentionally modify the number of frames.
+    FAX & JBIG write multiple frames, but read only one frame.
+  */
+  if ((!strcmp( "FAX", format )) ||
+      (!strcmp( "JBIG", format )) ||
+      (!strcmp( "MNG", format )) ||
+      (!strcmp( "PSD", format )) ||
+      (!strcmp( "PTIF", format )) )
+    check_for_added_frames = MagickFalse;
 
   /*
    * Read original image
@@ -176,6 +201,10 @@ int main ( int argc, char **argv )
   GetExceptionInfo( &exception );
   imageInfo->dither = 0;
   (void) strncpy( imageInfo->filename, infile, MaxTextExtent-1 );
+
+  if (!magick_info->adjoin && !check_for_added_frames)
+    (void) strcat( imageInfo->filename, "[0]" );
+
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                         "Reading image %s", imageInfo->filename);
   original = ReadImage ( imageInfo, &exception );
@@ -187,20 +216,25 @@ int main ( int argc, char **argv )
     }
   if ( original == (Image *)NULL )
     {
-      (void) printf ( "Failed to read original image %s\n", imageInfo->filename );
+      (void) printf ( "Failed to read original image %s\n",
+		      imageInfo->filename );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
     }
 
+  /*
+    Save first input file number of frames for later verifications.
+  */
+  original_frames=GetImageListLength(original);
+
   /*
    * Obtain original image size if format requires it
    */
   rows    = original->rows;
   columns = original->columns;
   size[0]='\0';
-  magick_info=GetMagickInfo(format,&exception);
-  if (magick_info && magick_info->raw)
+  if (magick_info->raw)
     FormatString( size, "%dx%d", columns, rows );
 
   /*
@@ -221,7 +255,8 @@ int main ( int argc, char **argv )
     }
   if ( blob == NULL )
     {
-      (void) printf ( "Failed to write BLOB in format %s\n", imageInfo->magick );
+      (void) printf ( "Failed to write BLOB in format %s\n",
+		      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -246,7 +281,8 @@ int main ( int argc, char **argv )
     }
   if ( original == (Image *)NULL )
     {
-      (void) printf ( "Failed to read image from BLOB in format %s\n",imageInfo->magick );
+      (void) printf ( "Failed to read image from BLOB in format %s\n",
+		      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -261,7 +297,8 @@ int main ( int argc, char **argv )
   (void) strncpy( original->magick, format, MaxTextExtent-1 );
   (void) strcpy( imageInfo->filename, "" );
   original->delay = 10;
-  blob = (char *) ImageToBlob ( imageInfo, original, &blob_length, &exception );
+  blob = (char *) ImageToBlob ( imageInfo, original, &blob_length,
+				&exception );
   if (exception.severity != UndefinedException)
     {
       CatchException(&exception);
@@ -271,7 +308,8 @@ int main ( int argc, char **argv )
   imageInfo->depth=original->depth;
   if ( blob == NULL )
     {
-      (void) printf ( "Failed to write BLOB in format %s\n", imageInfo->magick );
+      (void) printf ( "Failed to write BLOB in format %s\n",
+		      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -295,7 +333,8 @@ int main ( int argc, char **argv )
     }
   if ( final == (Image *)NULL )
     {
-      (void) printf ( "Failed to read image from BLOB in format %s\n",imageInfo->magick );
+      (void) printf ( "Failed to read image from BLOB in format %s\n",
+		      imageInfo->magick );
       (void) fflush(stdout);
       exit_status = 1;
       goto program_exit;
@@ -308,7 +347,12 @@ int main ( int argc, char **argv )
       /*
        * Check final output
        */
+      double
+	fuzz_factor = 0;
 
+      /*
+	Some formats are lossy.
+      */
       if ((!strcmp( "CIN", format ) && (QuantumDepth == 8)) ||
           (!strcmp( "CMYK", format )) ||
           (!strcmp( "GRAY", format )) ||
@@ -324,18 +368,78 @@ int main ( int argc, char **argv )
           (!strcmp( "YUV", format ))  ||
           (final->compression == JPEGCompression))
         fuzz_factor = 0.06;
-  
-      if ( !IsImagesEqual(original, final ) &&
-           (original->error.normalized_mean_error > fuzz_factor) )
-        {
-          (void) printf( "R/W file check for format \"%s\" failed: %u/%.6f/%.6fe\n",
-                         format,(unsigned int) original->error.mean_error_per_pixel,
-                         original->error.normalized_mean_error,
-                         original->error.normalized_maximum_error);
-          (void) fflush(stdout);
-          exit_status = 1;
-          goto program_exit;
-        }
+
+      {
+	Image
+	  *o,
+	  *f;
+	
+	unsigned long
+	  frame=0;
+
+	/*
+	  Verify that frame pixels are identical (or close enough).
+	*/
+	for (o=original, f=final, frame=0;
+	     ((o != (Image *) NULL) && (f != (Image *) NULL)) ;
+	     o = o->next, f = f->next, frame++)
+	  {
+	    printf("Checking frame %ld...\n",frame);
+	    if ( !IsImagesEqual(o, f ) &&
+		 (original->error.normalized_mean_error > fuzz_factor) )
+	      {
+		(void) printf( "R/W file check for format \"%s\" failed "
+			       "(frame = %ld): %.6f/%.6f/%.6fe\n",
+			       format,frame,
+			       original->error.mean_error_per_pixel,
+			       original->error.normalized_mean_error,
+			       original->error.normalized_maximum_error);
+		(void) fflush(stdout);
+		exit_status = 1;
+		goto program_exit;
+	      }
+	  }
+
+	if (check_for_added_frames)
+	  {
+	    /*
+	      Verify that reads from BLOB R/W #1 and BLOB R/W #2 did
+	      return the same number of frames.
+	    */
+	    if ((o != (Image *) NULL) && (f != (Image *) NULL))
+	      {
+		(void) printf("R/W file check for format \"%s\" failed due to "
+			      "differing number of returned frames (%ld vs %ld)\n",
+			      format,
+			      GetImageListLength(original),
+			      GetImageListLength(final));
+		exit_status = 1;
+		goto program_exit;
+	      }
+
+	    /*
+	      If format supports multiple frames, then we should expect
+	      that frames are not lost (or spuriously added) due to
+	      read/write of format.
+	    */
+	    if (magick_info->adjoin)
+	      {
+		unsigned long
+		  final_frames;
+
+		final_frames=GetImageListLength(final);
+		if (original_frames != final_frames)
+		  {
+		    (void) printf("R/W file check for format \"%s\" failed due "
+				  "to differing number of returned frames (%ld "
+				  "vs %ld) from original file\n",
+				  format,original_frames,final_frames);
+		    exit_status = 1;
+		    goto program_exit;
+		  }
+	      }
+	  }
+      }
     }
 
  program_exit:
@@ -345,7 +449,11 @@ int main ( int argc, char **argv )
   if (final)
     DestroyImageList( final );
   final = (Image*)NULL;
+  if (blob)
+    MagickFree(blob);
+  blob=0;
 
+  DestroyExceptionInfo(&exception);
   DestroyImageInfo( imageInfo );
   DestroyMagick();
 
